(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

open Common
open Automata

(* simple determinisation *)
let naive_dfa a =
  let m = a.SNFA.m in
  let z = Span.empty (Bdd.constant m Set.empty) in
  let t x = 
    Stats.incr "t^sharp calls";
    Set.fold (fun i acc -> Span.merge (Bdd.binary m Set.union) (a.SNFA.t i) acc) x z in
  let o x = Set.fold (fun i acc -> Bdd.dsj (a.SNFA.o i) acc) x Bdd.bot in
  SDFA.({ m; t; o })

(* determinisation + reindexing *)
let reindexed_dfa a =
  let m = Bdd.init() in
  let z = Span.empty (Bdd.constant m IntSet.empty) in
  let r = ref 0 in
  let g = ref [||] in
  let back = Hashtbl.create 57 in
  let rec f x = 
    try Hashtbl.find back x 
    with Not_found ->
      let i = !r in
      let t = !g in
      let l = Array.length t in
      Hashtbl.add back x i;
      if i = l then (
	g := Array.make (2*l+1) (Bdd.bot,z);
	Array.blit t 0 !g 0 i
      ); 
      incr r;
      Stats.incr "non deterministic states";
      !g.(i) <- (a.SNFA.o x, Span.map (Bdd.unary m f') (a.SNFA.t x));
      i
  and f' x = Set.int_map f x in
  let t x = 
    Stats.incr "t^sharp calls";
    IntSet.fold (fun i acc -> Span.merge (Bdd.binary m IntSet.union) (snd !g.(i)) acc) x z in
  let o x = IntSet.fold (fun i acc -> Bdd.dsj (fst !g.(i)) acc) x Bdd.bot in
  SDFA.({ m; t; o }), f'

