(*******************************************************************)
(*  This is part of SymbolicKAT, it is distributed under the       *)
(*  terms of the GNU Lesser General Public License version 3       *)
(*           (see file LICENSE for more details)                   *)
(*                                                                 *)
(*  Copyright 2014: Damien Pous. (CNRS, LIP - ENS Lyon, UMR 5668)  *)
(*******************************************************************)

(** Symbolic algorithms for language equivalence *)

open Common
open Automata

(** Symbolic equivalence check, using queues [Q] and up-to technique [R]  

candidates for [Q] include:
- fifo queues for breadth-first exploration ([Queues.BFS])
- lifo queues for depth-first exploration ([Queues.DFS])
- random queues for random exploration ([Queues.RFS])

candidates for [R] include:
- plain relations (or sets): [Upto.SET], for the plain algorithm, without up-to techniques
- disjoint set forests: [Upto.DSF], for using up to equivalence

*)
module Symbolic(Q: QUEUE)(R: Upto.T): sig
  val equiv: ('s,'o) sdfa -> 's -> 's -> ('o*'o*gstring_) option
end

(** Symbolic version of Hopcroft and Karp's algorithm, using queues
    [Q] (implicitely uses up to equivalence) *)
module SymbolicHK(Q: QUEUE): sig
  val equiv: ('s,'o) sdfa -> 's -> 's -> ('o*'o*gstring_) option
end
